@@ -34,7 +34,7 @@ class ListManager:
 		self._data = copy.deepcopy(entries)
 
 		explainer = str(_('\n Choose an object from the list, and select one of the available actions for it to execute'))
-		self._prompt = prompt + explainer if prompt else explainer
+		self._prompt = prompt if prompt else explainer
 
 		self._separator = ''
 		self._confirm_action = str(_('Confirm and exit'))
@@ -44,13 +44,18 @@ class ListManager:
 		self._base_actions = base_actions
 		self._sub_menu_actions = sub_menu_actions
 
-		self._last_choice = None
+		self._last_choice: Optional[str] = None
 
 	@property
-	def last_choice(self):
+	def last_choice(self) -> Optional[str]:
 		return self._last_choice
 
-	def run(self):
+	def is_last_choice_cancel(self) -> bool:
+		if self._last_choice is not None:
+			return self._last_choice == self._cancel_action
+		return False
+
+	def run(self) -> List[Any]:
 		while True:
 			# this will return a dictionary with the key as the menu entry to be displayed
 			# and the value is the original value from the self._data container
@@ -76,10 +81,11 @@ class ListManager:
 			elif choice.value in self._terminate_actions:
 				break
 			else:  # an entry of the existing selection was choosen
-				selected_entry = data_formatted[choice.value]
+				selected_entry = data_formatted[choice.value]  # type: ignore
 				self._run_actions_on_entry(selected_entry)
 
-		self._last_choice = choice
+		self._last_choice = choice.value  # type: ignore
+
 		if choice.value == self._cancel_action:
 			return self._original_data  # return the original list
 		else:
@@ -122,21 +128,29 @@ class ListManager:
 			self._data = self.handle_action(choice.value, entry, self._data)
 
 	def selected_action_display(self, selection: Any) -> str:
-		# this will return the value to be displayed in the
-		# "Select an action for '{}'" string
+		"""
+		this will return the value to be displayed in the
+		"Select an action for '{}'" string
+		"""
 		raise NotImplementedError('Please implement me in the child class')
 
-	def reformat(self, data: List[Any]) -> Dict[str, Any]:
-		# this should return a dictionary of display string to actual data entry
-		# mapping; if the value for a given display string is None it will be used
-		# in the header value (useful when displaying tables)
+	def reformat(self, data: List[Any]) -> Dict[str, Optional[Any]]:
+		"""
+		this should return a dictionary of display string to actual data entry
+		mapping; if the value for a given display string is None it will be used
+		in the header value (useful when displaying tables)
+		"""
 		raise NotImplementedError('Please implement me in the child class')
 
 	def handle_action(self, action: Any, entry: Optional[Any], data: List[Any]) -> List[Any]:
-		# this function is called when a base action or
-		# a specific action for an entry is triggered
+		"""
+		this function is called when a base action or
+		a specific action for an entry is triggered
+		"""
 		raise NotImplementedError('Please implement me in the child class')
 
-	def filter_options(self, selection :Any, options :List[str]) -> List[str]:
-		# filter which actions to show for an specific selection
+	def filter_options(self, selection: Any, options: List[str]) -> List[str]:
+		"""
+		filter which actions to show for an specific selection
+		"""
 		return options
